
/****************************************************************************/
/*                                                                          */
/*  Work on individual units (typed label value pairs). Fixed size. Is      */
/*  it worthwhile to make some variable in size ?                           */
/*                                                                          */
/****************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <idsa_internal.h>

IDSA_UNIT *idsa_unit_new(char *n, unsigned int type, char *s)
{
  IDSA_UNIT *u;
  unsigned int realsize;

  if ((type & IDSA_S_MASK) > IDSA_M_LONG) {
#ifdef TRACE
    fprintf(stderr, "idsa_unit_new(): type or length too long\n");
#endif
    return NULL;
  }
  realsize = idsa_unit_size(type);

#ifdef TRACE
  fprintf(stderr, "idsa_unit_new(): allocating %d bytes (instead of full %d)\n", realsize, sizeof(IDSA_UNIT));
#endif

  if (realsize > sizeof(IDSA_UNIT)) {
    return NULL;
  }
  u = malloc(realsize);
  if (u) {
#ifdef TRACE
    fprintf(stderr, "idsa_unit_new(): copying name %s\n", n);
#endif
    strncpy(u->u_name, n, IDSA_M_NAME - 1);
    u->u_name[IDSA_M_NAME - 1] = '\0';
    u->u_type = type;
    if (s) {
#ifdef TRACE
      fprintf(stderr, "idsa_unit_new(): copying payload\n");
#endif
      if (idsa_unit_scan(u, s)) {
	free(u);
	u = NULL;
      }
    }
  }
  return u;
}

void idsa_unit_copy(IDSA_UNIT * a, IDSA_UNIT * b)
{
  if (a->u_type == b->u_type) {
#ifdef TRACE
    fprintf(stderr, "idsa_unit_copy(): copying %d bytes\n", a->u_type & IDSA_S_MASK);
#endif
    strncpy(a->u_name, b->u_name, IDSA_M_NAME);
    memcpy(a->u_ptr, b->u_ptr, b->u_type & IDSA_S_MASK);
  }
}

IDSA_UNIT *idsa_unit_dup(IDSA_UNIT * u)
{
  IDSA_UNIT *result;

  result = malloc(u->u_type & IDSA_S_MASK);
  if (result) {
    memcpy(result, u, u->u_type & IDSA_S_MASK);
  }
  return result;
}

void idsa_unit_free(IDSA_UNIT * u)
{
#ifdef TRACE
  fprintf(stderr, "idsa_unit_free(): deleting unit <%p>\n", u);
#endif
  free(u);
}
